/*
 *
 * Given an accelerogram, rsc() computes ("structural") response spectra
 * by the method of:
 *
 *   %A N. C. Nigam
 *   %A P. C. Jennings
 *   %T Calculation of response spectra from strong-motion earthquake records
 *   %J Bull. Seismol. Soc. Am.
 *   %V 59
 *   %P 909-922
 *   %D 1969
 *
 * This is a direct transcription of their equations, assuming
 * constant digitizing interval, dt.
 *
 * RETURNS a structure containing the maximal displacement, velocity,
 * absolute accelerations, and "pseudovelocity", and the sample numbers
 * at which each occurs.  (Pseudovelocity is angular frequency times
 * displacement.)
 *
 * Note that everything is a double or int.
 *
 * AUTHOR:  John R. Evans
 *          USGS, MS-977
 *          Menlo Park CA 94025
 *          evans@andreas.wr.usgs.gov
 *
 * Written 26 Oct 1994.
 * 16 March 1995, unrecorded modifications.
 * 29 Feb 1996, corrected pseudovelocity calculation from ww*velocity
 *    to ww*displacement.  JREvans.
 *
 */

#include "rsc_result.h"

rsc_result
rsc(hh, ww, dt, aa, nn/*, xx, vv, zz*/)
	double	hh;	/* damping fracton (typically 0.05, i.e., 5%)	*/
	double	ww;	/* natural frequency of the oscillator
			   (2*PI/period)                         [Hz]	*/
	double	dt;	/* digitizing interval                   [s]	*/
	double	*aa;	/* pointer to accelerogram             [cm/s/s]	*/
#if 0
	double	*xx;	/* debug					*/
	double	*vv;	/* debug					*/
	double	*zz;	/* debug					*/
#endif
	int	nn;	/* number of sample points in accelerogram	*/
{
	rsc_result	result;		/* you guessed it		*/

	int	ii;	/* time pointer					*/

	double	a11, a12, a21, a22;	/* matrix A coefficients	*/
	double	b11, b12, b21, b22;	/* matrix B coefficients	*/

	double	xx_old = 0.0;	/* displacement at time ii-1		*/
	double	xx_new = 0.0;	/* displacement at time ii		*/
	double	vv_old = 0.0;	/* velocity at time ii-1		*/
	double	vv_new = 0.0;	/* velocity at time ii			*/
	double	aa_old;		/* acceleration at time ii-1		*/
	double	aa_new;		/* acceleration at time ii		*/
	double	zz_new;		/* absolute acceleration at time ii	*/

	double	sin(), cos(), sqrt(), exp(), fabs();

	/* Often-repeated portions of A and B coefficients ... */
	double cosbit, sinbit, hratio, expbit, blue, yellow;
	/* ... and even smaller bits. */
	double rootbit, wrootbit, w2, w3;

	/* Initialize result structure */
	result.dd = (double)0.0;
	result.di = -1;
	result.vv = (double)0.0;
	result.vi = -1;
	result.zz = (double)0.0;
	result.gg = (double)0.0;
	result.zi = -1;
	result.pv = (double)0.0;

	/* Compute coefficients of A and B matrices of Nigam and Jennings */

	/* Fist the small bits ... */

	w2 = ww * ww;
	w3 = ww * w2;

	rootbit = sqrt(1.0 - hh * hh);
	wrootbit = ww * rootbit;

	cosbit = cos(wrootbit * dt);
	sinbit = sin(wrootbit * dt);
	hratio = hh / rootbit;

	expbit = exp(-hh * ww * dt);
	blue = 2.0 * hh / (w3 * dt);
	yellow = (2.0 * hh * hh - 1.0) / (w2 * dt);

	/* ... then the coefficients. */

	a11 = expbit * (hratio * sinbit + cosbit);
	a12 = expbit * sinbit / wrootbit;
	a21 = -ww * expbit * sinbit / rootbit;
	a22 = expbit * (cosbit - hratio * sinbit);		/* (6d) */

	b11 = expbit * ((yellow + hh / ww) * (sinbit / wrootbit)
		+ (blue + 1.0 / w2) * cosbit) - blue;
	b12 = -expbit * (yellow * sinbit / wrootbit + blue * cosbit)
		- (1.0 / w2) + blue;
	b21 = expbit * ((yellow + hh / ww) * (cosbit - hratio * sinbit)
		- (blue + 1.0 / w2) * (wrootbit * sinbit + hh * ww * cosbit))
		+ 1.0 / (w2 * dt);
	b22 = -expbit * (yellow * (cosbit - hratio * sinbit)
		- blue * (wrootbit * sinbit + hh * ww * cosbit))
		- 1.0 / (w2 * dt);				/* (6e) */

	/* run along the accelerogram (beginning at SECOND point) */
#if 0
	xx[0] = (double)0.0; vv[0] = (double)0.0; zz[0] = (double)0.0;
#endif
	for (ii = 1 ; ii < nn ; (ii++, xx_old = xx_new, vv_old = vv_new)) {

		aa_old = aa[ii - 1];
		aa_new = aa[ii];

		xx_new = xx_old * a11 + vv_old * a12
			+ aa_old * b11 + aa_new * b12;

		vv_new = xx_old * a21 + vv_old * a22
			+ aa_old * b21 + aa_new * b22;	/* (6a-c) */

#if 0
		printf("%lf ", xx_old);
		printf("%lf ", vv_old);
		printf("%lf\n", aa_old);
#endif

#if 0
		printf("%lf ", xx_old * a11);
		printf("%lf ", vv_old * a12);
		printf("%lf ", aa_old * b11);
		printf("%lf ", aa_new * b12);

		printf("%lf ", xx_old * a21);
		printf("%lf ", vv_old * a22);
		printf("%lf ", aa_old * b21);
		printf("%lf\n", aa_new * b22);
#endif

		zz_new = -(2.0 * hh * ww * vv_new + w2 * xx_new); /* (7) */

#if 0
		xx[ii] = xx_new;
		vv[ii] = vv_new;
		zz[ii] = zz_new;
#endif

		if (fabs(xx_new) > result.dd) {
			result.dd = fabs(xx_new);
			result.pv = result.dd * ww;	/* pseudovelocity */
			result.di = ii;
		}						/* (8a) */

		if (fabs(vv_new) > result.vv) {
			result.vv = fabs(vv_new);
			result.vi = ii;
		}						/* (8b) */

		if (fabs(zz_new) > result.zz) {
			result.zz = fabs(zz_new);
			result.zi = ii;
		}						/* (8c) */
	}

	result.gg = result.zz / oneg;	/* also represent result in g's */

	return (result);
}
